/* global process, Buffer */

  /**
   * @namespace ZmqClient
   * Provides TCP Connection
   */

  var net = require("net");
  var util = require("util");
  var eventEmitter = require('events').EventEmitter;
//  var logClient = require('http');
//  var querystring = require('querystring');

  /*
   * The emitter emits the following messages'
   * 'open'	 - After signature handling
   * 'status'  - Frame debugging
   * 'connect' - After accepting peer's identity 
   * 'data'	 - On data handling
   * 'end'	 - On socket disconnect  
   * 'close'	 - On socket disconnect with error
   * 'error'	 - On error
   */


  //  ZMQ Signature 
  var ZMQSIGNATURE = new Buffer([0xFF,0,0,0,0,0,0,0,0,0x7F]);
  var ZMQSIGNATURE_VERMAJOR = new Buffer([03]);
  var ZMQSIGNREST = new Buffer([0,0x4E,0x55,0x4C,0x4C,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0]);
  //  ZMTP Handshake
  var ZMQHANDSHAKE = new Buffer([0x04,0x19,0x05,0x52,0x45,0x41,0x44,0x59,11,0x53,0x6F,0x63,0x6B,0x65,0x74,0x2D,0x54,0x79,0x70,0x65,0,0,0,0x03,0x53,0x55,0x42]);
  //  ZMTP Version
  var zmtpVersionMajor = 0;
  var zmtpVersionMinor = 0;
  var zmtpAsServer = false;

  var v_Errors = {
      IOERROR : 1,
      SECURITY_ERROR : 2
  };

  Object.keys(v_Errors).forEach(function(name){
    Object.defineProperty(ZmqClient, name, {
      enumerable: true,
      configurable: false,
      writable: false,
      value: v_Errors[name]
    });
  });


  //Constructor
  //public function ZmqSocket (identity: String = null)
  function ZmqClient(identity)
  {
      if (!(this instanceof ZmqClient)) {
          return new ZmqClient(identity);
      }

      /**
       * Post message to log server.
       * This is only for development.
       * @param message Message to log.
       * @param level 
       */
      var int_logger = function(message, level){
        if (typeof message !== 'string' || Number(message)!=='NaN'){
          try {
            message = JSON.stringify(message);
          } catch (e) {
          }
        }
        console.log(level, message);
      };

      process.on('uncaughtException', function(err) {
        //console.error(err);
        // handle the error safely
          //var stack = getStack(err.stack);

//          int_logger(JSON.stringify(stack[0], stack.slice(1)),'error');
        var stack = new Error().stack;
        int_logger({mesage:err.message, stack:stack},'error');
      });


      int_logger('constructor ['+identity+']','debug');

      // Instance of the caller object 
      var instance = null;
      //  Encapsulated tcp socket.
      var v_socket;
      //  Identity of socket, null if anonymous.
      var v_identity;
      //  Identity of peer, null if anonymous.
      var peerIdentity = null;
      //  True if received peer's identity.
      var connected = false;
      //  FIFO queue of received messages 
      var queue = []; 
      //  States that there are more frames of last message 
      var more = false;
      //  Current message, stored here until all frames are read
      var current = null;
      //  Number of bytes required for current decoder
      //  1 is value required by frameLengthDecoder
      var decoderMinBytes = 1;
      //  When decoderMinBytes are available, they are read
      //  and decoder is called. The decoder returns pointer
      //  to next decoder, and modifies decoderMinBytes accordingly.
      var decoder = frameLengthDecoder;
      //  Length of current frame
      var frameLength = 0;
      //  Flags of current frame
      var frameFlags = 0;
      //  Handshake command body
      var commandBody = null;

      //  Incoming Data buffer
      var dataBuffer = [];

      //  Outgoing Data Buffer
      var outDataBuffer = [];


      instance = this;
      eventEmitter.apply(this, arguments);
      //variables
      v_identity = identity;
      queue = [];

      //public function connect (host: String, port: int)
      this.connect = function (host, port){
          instance.emit('status', 'connect'); // For debugging

          v_socket = net.createConnection(port, host, connectHandler);
          v_socket.on('data', function(data){
              setTimeout(function(){
                  socketDataHandler(data);
              },0);
          });
          v_socket.on('end', function(){

              instance.logger('disconnected', 'debug');
              //v_socket = undefined;
              instance.emit('end');
          });
          v_socket.on('close', function(had_error) {
              instance.logger({msg:'disconnected',HadError: had_error},'debug');
              instance.emit('close', {had_error: had_error});
              //v_socket = undefined;
          });

          v_socket.on('error', function(er) {
              //console.log('Error connecting', er);
              v_socket = undefined;
              instance.emit('error', {errorCode: v_Errors.IOERROR, errorMessage:"Error connecting.", extra: er});
          });

          v_socket.on('drain',internalSocketSendData);
      };

      //public function close ()
      this.close = function(){
          instance.logger('In close', 'debug');
          if (v_socket){
            v_socket.end();
          }
      };
      //public function available (): Boolean
      this.available = function(){
          instance.logger('In available', 'debug');
          return queue.length > 0;
      };

      //public function recv (): Array
      this.recv = function(){
          //console.log("In recv","Queue is ", queue.join(' '));
          instance.logger({msg:'In recv.',  queue:queue.join(' ')}, 'debug');
          if (queue.length > 0)
          {
              var bin = queue.shift ();
              //console.log(" Bin is ", bin);
              var str = [];
  //            var char = 0;
              while (bin.length > 0){
                  var frame = bin.shift();
                  //console.log(" Frame is ", frame);
                  if (frame !== null){
                      var buf = new Buffer(frame);
                      str.push(buf.toString());
  //                    for (var i = 0, max = frame.length; i < max; i++) {
  //                        char = frame.shift();
  //                        if (char<32 || char > 127){
  //                            console.log("Got char ", char);
  //                        }
  //                        str += (String.fromCharCode(char));
  //                    }
                      //console.log(' Add frame. Str: ', str);
                  }else{
                      str.push('');
                      //console.log(' No frame. Str: ', str);
                  }
                  //console.log(" Value is ", str);
              }
              return str;
          }
          return null;
      };

      //public function recvBin (): Array 
      this.recvBin = function(){
          //console.log("In recvBin", queue);
          instance.logger({msg:'In recvBin.',  queue:queue.join(' ')}, 'debug');
          if (queue.length > 0){
              return queue.shift ();
          }
          return null;
      };

      //public function send (msg: Array): void
      this.send = function(msg){
          //console.log("In send");
          instance.logger('In send.', 'debug');
          if (!canSend (msg)){
              return;
          }

          while (msg.length > 0)
          {
                  var frame = msg.shift ();
                  writeFrame (frame, msg.length > 0);
          }
      };

      //public function sendBin (msg: Array): void
      this.sendBin = function(msg){
//	        console.log("In sendBin");
          instance.logger('In sendBin.', 'debug');
          if (!canSend (msg)){
              return;
          }

          while (msg.length > 0)
          {
                  var frame = msg.shift ();
                  writeFrame (frame, msg.length > 0);
          }
      };

      //public function isConnected (): Boolean
      /**
       * Check if ZmqClient is connected.
       * @returns {Boolean} True if connected else false.
       */
      this.isConnected = function(){
          return connected;
      };

      //public function subscribe (subject: String) : void
      this.subscribe = function(subject){
          //console.log("In subscribe");
          instance.logger('In subscribe.', 'debug');

          var strLen = subject.length;

          var buf = new Buffer(strLen+3);
          buf[0] = 0;
          buf[1] = strLen+1;
          buf[2] = 1; //Subscribe message
          buf.write(subject,3);

          //console.log('Buffer:', buf);
          instance.logger({msg:'In subscribe.',  buffer:Buffer(buf).toString()}, 'debug');
          internalSocketWrite(buf);

      };

      //public function unsubscribe (subject: String) : void
      this.unsubscribe = function(subject){
          //console.log("In unsubscribe");
          instance.logger('In unsubscribe', 'debug');
          var strLen = subject.length;

          var buf = new Buffer(strLen+3);
          buf[0] = 0;
          buf[1] = strLen+1;
          buf[2] = 0; //Unsubscribe message
          buf.write(subject,3);

          //console.log('Buffer:', buf);
          instance.logger({msg:'In unsubscribe.',  buffer:buf.toString()}, 'debug');
          internalSocketWrite(buf);

      };

      this.getIdentity = function(){
          return v_identity;
      };

      function internalSocketSendData(){
          setTimeout(function(){
              if (outDataBuffer.length>0){
                  var msg = outDataBuffer[0];
                  //console.log('Send Data', msg);
                  instance.logger({msg:'Send Data', data:msg}, 'debug');
                  v_socket.write(msg,true,function(){
                      outDataBuffer.shift();
                      internalSocketSendData();
                  });
              }
          },0);
      }

      function internalSocketWrite(data)
      {
          outDataBuffer.push(data);
          if (outDataBuffer.length<2){
              internalSocketSendData();
          }
      }

      /**
       * Not used
       * @param buffer
       * @returns {String}
       * @depricated
       */
      function bufferToArrayBuffer(buffer){
          var str = "" ;
          var charCode;
          for (var i = 0, max = buffer.length; i < max; i++) {
              charCode = buffer[i];
              if (charCode!==0){
                  str += String.fromCharCode(charCode);
              }
          }

          return str;
      }

      //private function canSend (msg: Array): Boolean
      function canSend(msg){
          //console.log("In canSend");
          instance.logger('In canSend', 'debug');
          if (!connected)
          {
              instance.emit('error', {errorCode: v_Errors.IOERROR, errorMessage:"Socket is not connected yet"});
              return false;
          }
          if (msg === null)
          {
              instance.emit('error', {errorCode: v_Errors.IOERROR, errorMessage:"Input message is NULL"});
              return false;
          }
          return true;    
      }

      //private function writeSignature(): void
      function writeSignature(){
          internalSocketWrite(ZMQSIGNATURE);
      }

      //private function writeFrame (frame: String, more: Boolean = false): void
      function writeFrame(frame, more){
          //console.log("In writeFrame");
          instance.logger('In writeFrame :'+frame, 'debug');
          var buf = new Buffer(frame);
      //    if (frame != null && frame.length > 0)
      //            buf.writeUTFBytes (frame);

          writeFrameBin (buf, more);
      }

      //private function writeFrameBin (frame: ByteArray, more: Boolean = false): void
      function writeFrameBin(frame, more){
          //console.log("In writeFrameBin");
          instance.logger('In writeFrameBin :'+frame, 'debug');
          //  Length
          var len = frame.length + 1;
          if (len < 255)
          {
                  internalSocketWrite(len);
          }
          else
          {
                  //  Flags 64-bit length 
                  internalSocketWrite (0xFF);
                  //  Leading 32 bits set to 0
                  internalSocketWrite (0);
                  //  Least significant 32 bits are our length
                  internalSocketWrite (len);
          }

          //  Flags
          if (more){
                  internalSocketWrite (1);
          } else {
                  internalSocketWrite (0);
          }

          //  Data
          if (frame.length > 0){
              internalSocketWrite (frame);
          }
      }

      //private function readNextFrame (): Boolean
      function readNextFrame(){
          //console.log('In readNextFrame', dataBuffer.join(), dataBuffer.length, decoderMinBytes);
          instance.logger({msg:'In readNextFrame',d:dataBuffer.join(), len:dataBuffer.length, decoderMinBytes:decoderMinBytes}, 'debug');
          //  Main decoding cycle 
          while (dataBuffer.length >= decoderMinBytes)
          {
              //console.log('[readNextFrame]',dataBuffer.join(), dataBuffer.length, decoderMinBytes);
              //instance.logger('[readNextFrame] '+dataBuffer.join()+ ', ' + dataBuffer.length + ', ' + decoderMinBytes, 'debug');
              instance.logger({msg:'[readNextFrame]',d:dataBuffer.join(), len:dataBuffer.length, decoderMinBytes:decoderMinBytes}, 'debug');
              //  Read bytes required by current decoder
              //var bytes = new Uint8Array();
              //v_socket.readBytes (bytes, 0, decoderMinBytes);
              // For debug only
              /*
              trace(bytes + " - "+ fromArray(bytes, true));
              */
              //dispatchEvent (new StatusEvent(StatusEvent.STATUS, false, false,fromArray(bytes, true),"status"));
              instance.emit('status',dataBuffer);

              //  Decode.
              if (decoder===undefined){
                  //console.log('Something is wrong. Decoder is not defined!');
                  instance.logger('Something is wrong. Decoder is not defined!', 'debug');
                  return;
              }
              //console.log("Decoder: "+decoder.toString().split('\n')[0]);
              instance.logger('Decoder: '+decoder.toString().split('\n')[0], 'debug');
              decoder = decoder (dataBuffer);

              //  Returned NULL means a new frame was read.
              if (decoder === null)
              {
                  //console.log('a new frame was read.');
                  instance.logger('a new frame was read.', 'debug');
                  //  Set initial decoder to read the next frame.
                  decoderMinBytes = 1;
                  decoder = frameFlagsDecoder;
                  //dataBuffer = [];
                  //  True means frame was read.
                  return true;
              }
          }

          //  No frame was read.
          //console.log("No frame");
          instance.logger('No frame.', 'debug');
          return false;
      }

      //private function signatureDecoder (bytes: ByteArray): Function
      function signatureDecoder(bytes){
          //console.log('In signatureDecoder', bytes);
          instance.logger('In signatureDecoder.', 'debug');
          // First byte is for detection of Version 1
          var sb = bytes.shift();
          if (sb !== 0xFF)
          {
      //        dispatchEvent(new IOErrorEvent(IOErrorEvent.IO_ERROR, false, false,
      //                "Version 1 ZMTP is not supported"));
              instance.emit('error', {errorCode: v_Errors.IOERROR, errorMessage:"Version 1 ZMTP is not supported."});
              v_socket.end();

              return null;
          }
          decoderMinBytes = 8;
          return signatureDecoderPadding;
      }

      //private function signatureDecoderPadding(bytes: ByteArray): Function
      function signatureDecoderPadding(bytes){
          //console.log('In signatureDecoderPadding', bytes);
          instance.logger('In signatureDecoderPadding.', 'debug');

          // Just throw this away
          var pads = bytes.shift();

          // Throw away this too
          pads = bytes.shift();

          decoderMinBytes = 1;
          return signatureDecoderLegacy;
      }

      //private function signatureDecoderLegacy(bytes: ByteArray): Function
      function signatureDecoderLegacy(bytes){
          //console.log('In signatureDecoderLegacy', bytes);
          instance.logger('In signatureDecoderLegacy.', 'debug');

          var sb = bytes.shift();
          var zmtpv1 = ((sb & 0x1) === 0x1);
          if (zmtpv1)
          {
                  instance.emit('error', {errorCode: v_Errors.IOERROR, errorMessage:"Version 1 ZMTP is not supported."});
                  v_socket.end();

                  return null;
          }
          internalSocketWrite(ZMQSIGNATURE_VERMAJOR);
          //v_socket.flush();
          dataBuffer = [];
          decoderMinBytes = 54;
          return signatureDecoderRest;
      }

      //private function signatureDecoderRest(bytes: ByteArray): Function
      function signatureDecoderRest(bytes){
          //console.log("In signatureDecoderRest", bytes.join());
          instance.logger('In signatureDecoderRest.', 'debug');

          var sb = bytes.shift();
          zmtpVersionMajor = sb;

          sb = bytes.shift();
          zmtpVersionMinor = sb;

          var secMechanism = "" ;
          var charCode;
          for (var i = 0, max = 20; i < max; i++) {
              charCode = bytes.shift();
              if (charCode!==0){
                  secMechanism += String.fromCharCode(charCode);
              }
          }
          if (secMechanism !== "NULL")
          {
                  instance.emit('error', {errorCode: v_Errors.IOERROR, errorMessage:"Security mechanism other than NULL is not supported"});
                  v_socket.end();

                  return null;
          }

          sb = bytes.shift();
          zmtpAsServer = (sb & 0x1) === 0x1;


          /* Ignore the rest 31 padding bytes*/
          dataBuffer = [];
          internalSocketWrite(ZMQSIGNREST);


      /* Perform handshake now */

          internalSocketWrite(ZMQHANDSHAKE);

          decoderMinBytes = 1;
          return signatureHandshake;
      }

      //private function signatureHandshake(bytes: ByteArray): Function
      function signatureHandshake(bytes){
          //console.log("In signatureHandshake", bytes.join());
          instance.logger('In signatureHandshake. ' + bytes.join(), 'debug');

          var sb = bytes.shift();
          if (sb !== 0x04) /* Expect command size else ton poulo! */
          {
                  instance.emit('error', {errorCode: v_Errors.IOERROR, errorMessage:"Sender did not send Handshake!"});
                  v_socket.end();

                  return null;
          }

          decoderMinBytes = 1;
          return signatureHandshakeLength;


      }

      //private function signatureHandshakeLength(bytes: ByteArray): Function
      function signatureHandshakeLength(bytes){
          //console.log("In signatureHandshakeLength", bytes.join());
          instance.logger('In signatureHandshakeLength. ' + bytes.join(), 'debug');

          var sb = bytes.shift();
          decoderMinBytes = sb;
          return signatureHandshakeCommand;
      }

      //private function signatureHandshakeCommand(bytes: ByteArray) : Function
      function signatureHandshakeCommand(bytes){
          //console.log("In signatureHandshakeCommand", bytes.join());
          instance.logger('In signatureHandshakeCommand. ' + bytes.join(), 'debug');
          if (commandBody === null){
              commandBody = [];
          }
          decoderMinBytes -= bytes.length;
          for (var i = 0, max = bytes.length; i < max; i++) {
              commandBody.push(bytes.shift());
          }

          if (decoderMinBytes <= 0)
          {
              //console.log("Command finished. What to do with it?");
              instance.logger('Command finished. What to do with it?', 'debug');
              /* Command finished. What to do with it? */
              connected = true;
              //dispatchEvent(new Event(OPEN));
              instance.emit('open');

              //dataBuffer = [];
              decoderMinBytes = 1;
              return frameFlagsDecoder;
          }

          return signatureHandshakeCommand;
      }

      //private function handshakeResponseDecoder (bytes: ByteArray): Function
      /**
       * @private
       * @param {ByteArray} data
       * @depricated
       */
      function handshakeResponseDecoder(data){
          //console.log("In handshakeResponseDecoder");
          instance.logger('In handshakeResponseDecoder', 'debug');

          //var sb = 
          data.shift();
          decoderMinBytes = 1;
          return handshakeResponseLengthDecoder;
      }

      //private function handshakeResponseLengthDecoder (bytes: ByteArray): Function
      function handshakeResponseLengthDecoder(data){
          //console.log("In handshakeResponseLengthDecoder");
          instance.logger('In handshakeResponseLengthDecoder', 'debug');
          var sb = data.shift();
          decoderMinBytes = sb;
          return handshakeResponseBodyDecoder;
      }

      //private function handshakeResponseBodyDecoder (bytes: ByteArray): Function
      /**
       * @private
       * @param {ByteArray} bytes
       * @depricated
       */
      function handshakeResponseBodyDecoder(bytes){
          //console.log("In handshakeResponseBodyDecoder");
          instance.logger('In handshakeResponseBodyDecoder', 'debug');
          decoderMinBytes = 1;
          return frameLengthDecoder;
      }

      //private function frameLengthDecoder (bytes: ByteArray): Function
      function frameLengthDecoder (data){
          //console.log('In frameLengthDecoder', data.join(' '));
          instance.logger('In frameLengthDecoder : '+ data.toString(), 'debug');
          //  First byte is length.
          //frameLength = bytes[0];
          frameLength = data.shift();

          //  Length == 0 is bad
          if (frameLength === 0)
          {
              /* Version 3 can send an empty frame */
              /*
               dispatchEvent (new IOErrorEvent (IOErrorEvent.IO_ERROR, false, false, 
               "Socket is sending bad length"));
               socket.close ();

               return null;
               */
              return frameBodyDecoder(null);
              //decoderMinBytes = 1;
              //return frameFlagsDecoder;
              //return null;

          }

          decoderMinBytes = frameLength;
          return frameBodyDecoder;
      }

      //private function frameBigLengthDecoder (bytes: ByteArray): Function
      function frameBigLengthDecoder(data){
          //console.log("In frameBigLengthDecoder");
          instance.logger('In frameBigLengthDecoder.', 'debug');

          //frameLength = data.shift();
          frameLength = new Buffer(data.splice(0,4)).readUInt32BE(0);

          //  If significant 32-bits are not 0, means message
          //  size exceeds maximum, and we close connection.
          if (frameLength !== 0)
          {
              instance.emit('error', {errorCode: v_Errors.IOERROR, errorMessage:"Incoming message size is too big"});

              v_socket.end ();
              return null;
          }

          //  Least significant 32 bits are the length.
          //frameLength = data.readUnsignedInt ();
          frameLength = new Buffer(data.splice(0,4)).readUInt32BE(0);

          if (frameLength === 0)
          {
              instance.emit('error', {errorCode: v_Errors.IOERROR, errorMessage:"Socket is sending bad length (0)"});
              v_socket.end ();

              return null;
          }

          decoderMinBytes = frameLength;
          return frameBodyDecoder;
      }

      //private function frameFlagsDecoder (bytes: ByteArray): Function
      function frameFlagsDecoder (data){
          //console.log('In frameFlagsDecoder',data.join(' '));
          instance.logger('In frameFlagsDecoder : ' + data.toString(), 'debug');

          //frameFlags = data[0];
          frameFlags = data.shift();
          more = (frameFlags & 0x1) === 0x1;
          var long = (frameFlags & 0x2) === 0x2;
          var command = (frameFlags & 0x4) === 0x4;

          if (command) {
              if (long)
              {
                  decoderMinBytes = 8;
                  return commandBigSizeDecoder;
              } else
              {
                  decoderMinBytes = 1;
                  return commandSizeDecoder;
              }
          } else
          {
              /* It is a frame */
              if (long) {
                  decoderMinBytes = 8;
                  return frameBigLengthDecoder;
              } else {
                  decoderMinBytes = 1;
                  return frameLengthDecoder;
              }
          }
      }

      //private function frameBodyDecoder (bytes: ByteArray): Function
      function frameBodyDecoder(data){
          if (data!==null){
              //console.log('In frameBodyDecoder', data.join(' '));
              instance.logger('In frameBodyDecoder : ' + data.toString(), 'debug');
          }else{
              //console.log('In frameBodyDecoder. Data is null.');
              instance.logger('In frameBodyDecoder. Data is null', 'debug');
          }
          if (current === null){
              current = [];
          }

          if (data!==null){
              current.push(data.splice(0, frameLength));
          }
          console.log('Current:', current.join(' '));
          instance.logger('Current:' + current.join(' '), 'debug');


          //  If it's the last frame of message, push current message to queue.
          if (!more)
          {
              queue.push(current);
              current = null;
              //console.log("Push to queue! Empty current");
              instance.logger('Push to queue! Empty current.', 'debug');

              dataBuffer=[];
              return null;
          } else
          {
              //  Reset decoder to read next frame.
              //console.log("Reset decoder to read next frame.");
              instance.logger('Reset decoder to read next frame.', 'debug');
              decoderMinBytes = 1;
              return frameFlagsDecoder;
          }

      }

      //private function commandSizeDecoder (bytes: ByteArray): Function

      function commandSizeDecoder(bytes){
          //console.log("In commandSizeDecoder");
          instance.logger('In commandSizeDecoder.', 'debug');
          decoderMinBytes = 1;
          return frameLengthDecoder;
      }

      //private function commandBigSizeDecoder (bytes: ByteArray): Function
      function commandBigSizeDecoder(bytes){
          //console.log("commandBigSizeDecoder is not implemented");
          instance.logger('commandBigSizeDecoder is not implemented.', 'warn');
          decoderMinBytes = 1;
          return frameLengthDecoder;
      }

      //private function connectHandler (event: Event): void
      function connectHandler (event){
          //console.log('In connectHandler');
          instance.logger('In connectHandler.', 'debug');

          //  Reset decoder.
          decoderMinBytes = 1;
          if (connected){
                  decoder = frameLengthDecoder;
          }else{
                  decoder = signatureDecoder;
          }
          //  Now we must send greeting
          // writeFrame (identity);
          writeSignature();
      }

      //private function socketDataHandler (event: ProgressEvent): void 
      function socketDataHandler(data){
          //console.log('In socketDataHandler Frame');
          instance.logger('In socketDataHandler.', 'debug');

          //console.log("Got:", data.length);

          // Hack the buggy connection issue
//          if (data.length===53 && data[0] !== 3){
//            instance.logger('0MQ Buggy connection detected. Hack Applied!', "debug");
//            dataBuffer.push(3);
//          }


          for (var i = 0, max = data.length; i < max; i++) {
              dataBuffer.push(data[i]);
          }

          //TODO : Enabling this will drive to logical errors. Why? 
//		    if (dataBuffer.length<=decoderMinBytes){
//		    	instance.logger('In socketDataHandler. dataBuffer.lenght<=decoderMinBytes', 'debug');
//		        return;
//		    }

          //  If frame hasn't arrived yet, just wait for more data.
          if (!readNextFrame())
              return;

          //  If we haven't got peer's identity yet, read it now.
          if (!connected)
          {
              //  If peer says that identity has more than 1 frame, than he's bad.
              if (more)
              {
                  //dispatchEvent (new IOErrorEvent (IOErrorEvent.IO_ERROR, false, false, "Peer has malformed identity"));
                  this.emit('error', new Error({errorCode: v_Errors.IOERROR, errorMessage: "Peer has malformed identity"}));
                  v_socket.end();
                  return;
              } else
              {

                  //  Accept peer's identity
                  current = queue.shift();
                  if (current.length > 0)
                      peerIdentity = current.shift();
                  connected = true;
                  //dispatchEvent (new Event (OPEN));
                  instance.emit('connect');

              }
          } else
          {
              if (!more) {
                  //dispatchEvent (new Event (MESSAGE));
                  instance.emit('data');
              }
          }
      }

      ZmqClient.prototype.logger = int_logger;
  }

  //Make the ZmqClient to extend eventEmitter.
  //ZmqClient.prototype.__proto = eventEmitter.prototype;
  util.inherits(ZmqClient, eventEmitter);


  //What this module exports goes here.
  module.exports.ZmqClient = ZmqClient;

  //module.exports.ZmqClientType = "Test";

