/* global XMR */

/**
 * XMR Connection
 * 
 * Emits
 * connected - on successful connection
 * dicconnected - on disconnect
 * and many more....
 */



const crypto = require("crypto");
const NodeRSA = require('./packages/node-rsa');
const rc4 = require('./rc4').rc4;

var Zmqclient = require('./ZmqClient.js');
var EventEmitter = require('events');
var eventEmitter = new EventEmitter();

eventEmitter.on('error', function(){
  //Do not do anything. It is just a hack for preventing defaults errors to throw!
});

var client = undefined;

/* Key management */
var myRSAkey;
// Create Key
function createKey(){
 
  myRSAkey = new NodeRSA({b:1024});
  myRSAkey.setOptions({encryptionScheme: 'pkcs1'});
 
  priv = myRSAkey.exportKey('private');
  pub = myRSAkey.exportKey('public');
  var kc = {private:priv, public:pub};
  console.log(kc);
  return kc;
};

function exportKey(){
  if (!myRSAkey){
    eventEmitter.emit('error', 'No key available');
    return "undefined";
  }
  var priv = myRSAkey.exportKey('private');
  var pub = myRSAkey.exportKey('public');
//  console.log(priv);
//  console.log(pub);
  return {private:priv, public:pub};
};


function getHexBuffer(hexStr){
  if (hexStr.length%2===1){
    hexStr = '0'+hexStr;
  }
  return Buffer(hexStr,"hex");
};

function int_importKeyComponents(key){
  try {
    var _n = getHexBuffer(key.n,"hex");
  } catch (e) {
    console.error(e);
    eventEmitter.emit('error', {
      message:"Could not create key n component", 
      n:key.n,
      error:e.toString(),
      extra:e
    });
  }
//  var _e = parseInt(key.e, 16);
  try {
    var _d = getHexBuffer(key.d,"hex");
  } catch (e) {
    console.error(e);
    eventEmitter.emit('error', {
      message:"Could not create key d component", 
      d:key.d,
      error:e.toString(), 
      extra:e
    });
  }
  try {
    var _p = getHexBuffer(key.p,"hex");
  } catch (e) {
    console.error(e);
    eventEmitter.emit('error', {
      message:"Could not create key p component", 
      p:key.p,
      error:e.toString(), 
      extra:e
    });
  }
  try {
    var _q = getHexBuffer(key.q,"hex");
  } catch (e) {
    console.error(e);
    eventEmitter.emit('error', {
      message:"Could not create key q component", 
      q:key.q,
      error:e.toString(), 
      extra:e
    });
  }
  try {
    var _dmp1 = getHexBuffer(key.dmp1,"hex");
  } catch (e) {
    console.error(e);
    eventEmitter.emit('error', {
      message:"Could not create key dmp1 component", 
      dmp1:key.dmp1,
      error:e.toString(), 
      extra:e
    });
  }
  try {
    var _dmq1 = getHexBuffer(key.dmq1,"hex");
  } catch (e) {
    console.error(e);
    eventEmitter.emit('error', {
      message:"Could not create key dmq1 component", 
      dmq1:key.dmq1,
      error:e.toString(), 
      extra:e
    });
  }
  try {
    var _coeff = getHexBuffer(key.coeff,"hex");
  } catch (e) {
    console.error(e);
    eventEmitter.emit('error', {
      message:"Could not create key coeff component", 
      coeff:key.coeff,
      error:e.toString(), 
      extra:e
    });
  }

  try {
    k = {"n": _n, "e": 65537, "d": _d, "p": _p, "q": _q, "dmp1": _dmp1, "dmq1": _dmq1, "coeff": _coeff};
    myRSAkey.importKey(k,'components');
  } catch (e) {
    console.error(e);
    eventEmitter.emit('error', {message:"Could not import key components", extra:e});
  }
};

/**
 * Import an existing key
 * @param {String|Object} key The key in PKCS1 private pem format or as components object
 * @returns {undefined}
 */
function importKey(key){
//  try {
    if (typeof (key) === ' undefined'){
      eventEmitter.emit('error', 'No import key data available');
      return;
    }
    if (!myRSAkey){
      myRSAkey = new NodeRSA();
      myRSAkey.setOptions({encryptionScheme: 'pkcs1'});
    }
    if (typeof(key) === 'string'){
      if (key.startsWith('---')){
        try {
          myRSAkey.importKey(key,'pkcs1-private-pem');
        } catch (e) {
          console.error(e);
          eventEmitter.emit('error', {message:"Could not import pem key", m: e.message, extra:e});
          return;
        }
      }else{
        try {
          var pkey = JSON.parse(key);
          int_importKeyComponents(pkey);
        } catch (e) {
          console.error(e);
          eventEmitter.emit('error', {message:"Could not parse key components", m: e.message, extra:e});
          return;
        }
      }
    }else{
      int_importKeyComponents(key);
    }
    var priv = myRSAkey.exportKey('private');
    var pub = myRSAkey.exportKey('public');
    console.log(priv,pub);
    return {private:priv, public:pub};
//  } catch (e) {
//    console.error(e);
//    eventEmitter.emit('error', {
//      message:"Could not export private and public keys", 
//      m: e.message, 
//      key:key
//    });
//    return;
//  }
};

/**
 * Decrypt message from xmr
 * @param {String} chipher Base64 message to decrypt
 * @param {String} key Base64 key
 * @returns {String} The unencrypted message
 */
function decrypt(chipher, key){
  var key_buf = Buffer(key, 'base64');
  
//  try {
    var rsa_decrypted = myRSAkey.decrypt(key_buf);
//  } catch (e) {
//    console.error(e);
//    eventEmitter.emit('error', {message:"Could not decrypt", m:e.message, extra:e});
//    return;
//  }
  
  var rc4_toBfr = Buffer(chipher, 'base64');
  
  rc4_ret = rc4(
        rc4_toBfr,
        rsa_decrypted
    );
  
  
  return rc4_ret;
}

// 0MQ client functions


// <editor-fold desc="0MQ client events">
var zmq_onData = function(){
  console.log('Got data');
  if (client.available()){
    dt=client.recv();
    //console.log(dt);
    if (dt.length === 3 ){
      try {
        var msg = decrypt(dt[2], dt[1]);
        console.log('xmrclient:data',msg);
        eventEmitter.emit('data', msg);
      } catch (e) {
        console.error('xmrclient:Could not decrypt data', dt);
        //eventEmitter.emit('data', dt);
        var r = {
          message:"Could not decrypt", 
          extra:e.message,
          data:dt
        };
//        if (e.stack){
//          r.stack = e.stack;
//        }
        
        eventEmitter.emit('error', r);
      }
    }else{
      eventEmitter.emit('data', dt);
    }
  }
};
var zmq_onError = function(e){
  console.error('xmrclient:error',e, e.message);
  eventEmitter.emit('error', e);
  //detach_events();
};
var zmq_onClose = function(){
  console.log('Connection closed.');
  eventEmitter.emit('disconnected');
  detach_events();
};
var zmq_onDisconnected = function(){
  console.log('Connection disconnected.');
  eventEmitter.emit('disconnected');
  detach_events();
};
var zmq_onOpen = function(){
  eventEmitter.emit('connected');
};
function attach_events(c){
  c.addListener('data', zmq_onData);
  c.addListener('error', zmq_onError);
  c.addListener('close', zmq_onClose);
  c.addListener('disconnected', zmq_onDisconnected);
  c.addListener('open', zmq_onOpen);
};
function detach_events(c){
  if (c){
    c.removeAllListeners();
  }
  client = undefined;
};
// </editor-fold>

/**
 * Connect to host
 * @param {String} host
 * @param {Number} port
 * @returns {undefined}
 */
function connect(host, port){
  if (!host){
    eventEmitter.emit('error', 'XMR host not provided');
    return false;
  }
  
  if (!port){
    eventEmitter.emit('error', 'XMR port not provided');
    return false;
  }
  
  if (client && client.isConnected){
    client.close();
    detach_events(client);
  }
  console.log('Connect: Create new 0MQ client');
  client = Zmqclient.ZmqClient();
  attach_events(client);
  client.connect(host, port);
  
  return true;
};
function disconnect(){
  if (client){
    client.close();
  }
  client = undefined;
  //eventEmitter.emit('disconnected');
};

function connection_status(){
  if (!client){
    return false;
  }
  return client.isConnected();
}
function subscribe(data){
  if (!client){
    eventEmitter.emit('error', 'Not connected');
    return;
  }
  client.subscribe(data);
  eventEmitter.emit('subscribed', data);
};
function unsubscribe(data){
  if (!client){
    eventEmitter.emit('error', 'Not connected');
    return;
  }
  client.unsubscribe(data);
  eventEmitter.emit('unsubscribed',data);
  //throw new Error('Not implemented!');
};

eventEmitter.connect = connect;
eventEmitter.disconnect = disconnect;
eventEmitter.connection_status = connection_status;
eventEmitter.subscribe = subscribe;
eventEmitter.unsubscribe = unsubscribe;
eventEmitter.createKey = createKey;
eventEmitter.exportKey = exportKey;
eventEmitter.importKey = importKey;
eventEmitter.decrypt = decrypt;

module.exports = eventEmitter;
